/*
 * Copyright (c) 2013 Mattes Groeger
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#import <Foundation/Foundation.h>

@class MGBenchmarkSession;

@protocol MGBenchmarkTarget <NSObject>

@optional

/**
* Will be called as soon as the session started, so the target can access
* additional data from the session (e.g. the `name` or `stepCount`).
*/
- (void)sessionStarted:(MGBenchmarkSession *)session;

/**
* Gets called for in-between steps in a benchmark session. The time since
* the last step (if any) or since the session start is passed in.
*/
- (void)passedTime:(NSTimeInterval)passedTime forStep:(NSString *)stepName;

/**
* This method is called when the total time since session start was
* measured. It gets passed in as `passedTime`.
*/
- (void)totalTime:(NSTimeInterval)passedTime;

@end